@@ -1,14 +1,17 @@
 #!/usr/bin/perl
+# -*- mode: cperl; indent-tabs-mode: t; tab-width: 3; cperl-indent-level: 3; -*-
 BEGIN { $| = 1; }
 use warnings;
 use strict;
+use List::Util qw(max);
+use File::Basename;
 use File::Spec;
 use Getopt::Long;
 use Digest::SHA qw(sha1_hex);
 Getopt::Long::Configure('no_ignore_case');
 
 # This is updated by the update-revision.pl script.
-my $revision = 13891;
+my $revision = 13897;
 
 # Generate list with:
 # vislcg3 --help 2>&1 | perl -wpne 'if (/^ / && /-(\w), --([-\w]+)/) {print "$2|$1=s\n"} elsif (/^ / && /--([-\w]+)/) {print "$1=s\n"} s/^.*$//s;' | perl -wpne 's/^/"/; s/$/",/;'
@@ -98,7 +101,40 @@ while (my ($k,$v) = each(%h)) {
 	}
 }
 
-my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($h{'grammar'});
+sub file_get_contents {
+	my ($fname) = @_;
+	local $/ = undef;
+	open FILE, '<:encoding(UTF-8)', $fname or die "Could not open ${fname}: $!\n";
+	my $data = <FILE>;
+	close FILE;
+	return $data;
+}
+
+sub mtime_rec {
+	my ($fn) = @_;
+	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($fn);
+	my $g = file_get_contents($fn);
+	$g =~ s@#[^\n]*\n@\n@g;
+	#print STDERR "DEBUG: $fn mtime $mtime\n";
+
+	while ($g =~ m@(?:^|\n|;)INCLUDE(?:\s+STATIC)?\s+(.+?)\s+;@gsi) {
+		my $ifn = $1;
+		#print STDERR "DEBUG: Including $ifn (pre-expand)\n";
+		if ($ifn =~ m@[~*\$]@) {
+			$ifn = glob($ifn);
+		}
+		if ($ifn !~ m@^/@) {
+			$ifn = dirname($fn).'/'.$ifn;
+		}
+		#print STDERR "DEBUG: Including $ifn\n";
+		$mtime = max($mtime, mtime_rec($ifn));
+	}
+
+	return $mtime;
+}
+
+my $mtime = mtime_rec($h{'grammar'});
+#print STDERR "DEBUG: mtime $mtime\n";
 
 my $bn = File::Spec->tmpdir()."/".sha1_hex($args.$h{'grammar'}).".".$revision.".".$mtime.".cg3b";
 
